from setuptools import setup

setup(name='noisemix',
      version='0.1',
      description='NoiseMix is a library for data generation for text datasets.',
      url='https://gitlab.com/hetazotutyun/NoiseMix',
      author='Anooshik Vartnian',
      author_email='anooshik_vartanian@edu.aua.com',
      license='MIT',
      packages=['noisemix'],
      zip_safe=False)